@@ -1,12 +1,10 @@
-"""
-OrderedGlobalDeclarations orders all global functions.
-"""
+""" OrderedGlobalDeclarations orders all global functions. """
 
 from pythran.analyses.aliases import StrictAliases
 from pythran.analyses.global_declarations import GlobalDeclarations
 from pythran.passmanager import ModuleAnalysis
 
-import ast
+import gast as ast
 
 
 class OrderedGlobalDeclarations(ModuleAnalysis):
@@ -23,26 +21,26 @@ class OrderedGlobalDeclarations(ModuleAnalysis):
 
     def visit_Name(self, node):
         if node in self.strict_aliases:
-            for alias in self.strict_aliases[node].aliases:
+            for alias in self.strict_aliases[node]:
                 if isinstance(alias, ast.FunctionDef):
                     self.result[self.curr].add(alias)
                 elif isinstance(alias, ast.Call):  # this is a bind
-                    for alias in self.strict_aliases[alias.args[0]].aliases:
+                    for alias in self.strict_aliases[alias.args[0]]:
                         self.result[self.curr].add(alias)
 
-    def run(self, node, ctx):
+    def run(self, node):
         # compute the weight of each function
         # the weight of a function is the number functions it references
-        super(OrderedGlobalDeclarations, self).run(node, ctx)
+        result = super(OrderedGlobalDeclarations, self).run(node)
         old_count = -1
         new_count = 0
         # iteratively propagate weights
         while new_count != old_count:
-            for k, v in self.result.iteritems():
-                [v.update(self.result[f]) for f in list(v)]
+            for v in result.values():
+                [v.update(result[f]) for f in list(v)]
             old_count = new_count
-            new_count = reduce(lambda acc, s: acc + len(s),
-                               self.result.itervalues(), 0)
+            new_count = sum(len(value) for value in result.values())
         # return functions, the one with the greatest weight first
-        return sorted(self.result.iterkeys(), reverse=True,
-                      key=lambda s: len(self.result[s]))
+        self.result = sorted(self.result.keys(), reverse=True,
+                             key=lambda s: len(self.result[s]))
+        return self.result
