@@ -27,17 +27,17 @@ namespace functor
 
     template <typename... T>
     auto operator()(T &&... args) const -> typename std::enable_if<
-        utils::all_of<!types::is_numexpr_arg<typename std::remove_cv<
-            typename std::remove_reference<T>::type>::type>::value...>::value,
+        !types::valid_numexpr_parameters<
+            typename std::decay<T>::type...>::value,
         decltype(NUMPY_NARY_FUNC_SYM(std::forward<T>(args)...))>::type;
 
     template <class... E>
     typename std::enable_if<
-        types::valid_numexpr_parameters<E...>::value,
+        types::valid_numexpr_parameters<typename std::decay<E>::type...>::value,
         types::numpy_expr<
             NUMPY_NARY_FUNC_NAME,
             typename types::NUMPY_NARY_RESHAPE_MODE<E, E...>::type...>>::type
-    operator()(E const &... args) const;
+    operator()(E &&... args) const;
     friend std::ostream &operator<<(std::ostream &os, NUMPY_NARY_FUNC_NAME)
     {
       return os << STR(NUMPY_NARY_FUNC_NAME);
